﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.ServiceModel;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace HeuristicLab.Services.Hive.SelfHost {
  public partial class JanitorActivatorForm : Form {
    private static readonly Uri BaseAddress = new Uri(@"http://localhost/Hive-3.3/HiveService.svc");

    private IEventManager eventManager {
      get { return ServiceLocator.Instance.EventManager; }
    }

    private IStatisticsGenerator statisticsGenerator {
      get { return ServiceLocator.Instance.StatisticsGenerator; }
    }

    private Thread janitorThread;

    private bool janitorRunning;

    private readonly EventWaitHandle janitorWaitHandle;

    private readonly ServiceHost host;

    private bool serviceRunning;

    public JanitorActivatorForm() {
      InitializeComponent();
      janitorFrequency_ValueChanged(null, null);

      janitorWaitHandle = new AutoResetEvent(false);

      host = new ServiceHost(typeof(HiveService), BaseAddress);
    }

    private void activateJanitorButton_Click(object sender, EventArgs e) {
      activateJanitorButton.Enabled = false;

      try {
        janitorRunning = !janitorRunning;

        if (janitorRunning && janitorThread == null) {
          janitorThread = new Thread(RunEventManager) {
            IsBackground = true
          };
          janitorThread.Start();
        }

        janitorWaitHandle.Set();
      }
      catch (Exception exc) {
        WriteLine(exc.Message);
        WriteLine(exc.StackTrace);
        janitorRunning = false;
        janitorThread = null;
      }

      WriteLine(janitorRunning ? "Activated Janitor" : "Janitor deactivated");

      activateJanitorButton.Text = janitorRunning ? "Stop Janitor" : "Start Janitor";

      activateJanitorButton.Enabled = true;
    }

    private void RunEventManager() {
      while (janitorRunning) {
        int frequency = 0;
        Invoke(new Action(() => frequency = janitorFrequency.Value));

        janitorWaitHandle.WaitOne(frequency * 1000);

        WriteLine("Run Janitor");
        lock (eventManager) {
          eventManager.Cleanup();
        }
        lock (statisticsGenerator) {
          statisticsGenerator.GenerateStatistics();
        }
      }
      janitorThread = null;
    }

    private async void runServiceButton_Click(object sender, EventArgs e) {
      runServiceButton.Enabled = false;

      try {
        await StartService();

        WriteLine(serviceRunning ? "Service is ready at " + BaseAddress : "Service is closed");
      }
      catch (Exception exc) {
        WriteLine(exc.Message);
        WriteLine(exc.StackTrace);
        serviceRunning = false;
      }

      runServiceButton.Text = serviceRunning ? "Stop Service" : "Start Service";
      runServiceButton.Enabled = true;
    }

    private Task StartService() {
      return Task.Run(() => {
        serviceRunning = !serviceRunning;

        if (serviceRunning) {
          host.Open();
        } else {
          host.Close();
        }
      });
    }

    private async void runJanitorButton_Click(object sender, EventArgs e) {
      runCleanupButton.Enabled = false;

      await Task.Run(() => {
        lock (eventManager) {
          WriteLine("Run Cleanup");
          eventManager.Cleanup();
        }
      });

      runCleanupButton.Enabled = true;
    }

    private async void generateStatisticsButton_Click(object sender, EventArgs e) {
      generateStatisticsButton.Enabled = false;

      await Task.Run(() => {
        lock (eventManager) {
          WriteLine("Generate Statistics");
          statisticsGenerator.GenerateStatistics();
        }
      });

      generateStatisticsButton.Enabled = true;
    }

    private void janitorFrequency_ValueChanged(object sender, EventArgs e) {
      frequencyTextBox.Text = string.Format("{0} sek", janitorFrequency.Value);
    }

    private void WriteLine(string msg) {
      if (InvokeRequired) {
        Invoke(new Action<string>(WriteLine), msg);
        return;
      }
      outputTextbox.AppendText(string.Format("{0}: {1}{2}", DateTime.Now.ToLongTimeString(), msg, Environment.NewLine));
    }
  }
}
